@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2005 Silicon Graphics, Inc.
+ * Copyright (c) 2000-2006 Silicon Graphics, Inc.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -38,14 +38,6 @@
 #include <sys/fs/xfs_fsops.h>
 #include <sys/fs/xfs_itable.h>
 
-#define __s8		char
-#define __s16		short
-#define __s32		__int32_t
-#define __s64		__int64_t
-#define __u8		unsigned char
-#define __u16		unsigned short
-#define __u32		__uint32_t
-#define __u64		__uint64_t
 #define __int8_t	char
 #define __int16_t	short
 #define __uint8_t	unsigned char
@@ -125,17 +117,65 @@ typedef struct xfs_fsop_attrmulti_handlereq {
 	struct xfs_attr_multiop		*ops; /* attr_multi data */
 } xfs_fsop_attrmulti_handlereq_t;
 
+/* start doing packed stuctures here */
+#define HAVE_FORMAT32	1
+#pragma pack 1
+typedef struct xfs_inode_log_format_32 {
+	__u16			ilf_type;	/* inode log item type */
+	__u16			ilf_size;	/* size of this item */
+	__u32			ilf_fields;	/* flags for fields logged */
+	__u16			ilf_asize;	/* size of attr d/ext/root */
+	__u16			ilf_dsize;	/* size of data/ext/root */
+	__u64			ilf_ino;	/* inode number */
+	union {
+		__u32		ilfu_rdev;	/* rdev value for dev inode*/
+		uuid_t		ilfu_uuid;	/* mount point value */
+	} ilf_u;
+	__s64			ilf_blkno;	/* blkno of inode buffer */
+	__s32			ilf_len;	/* len of inode buffer */
+	__s32			ilf_boffset;	/* off of inode in buffer */
+} xfs_inode_log_format_32_t;
+
+typedef struct xfs_extent_32 {
+	__u64	ext_start;
+	__u32	ext_len;
+} xfs_extent_32_t;
+
+typedef struct xfs_efi_log_format_32 {
+	__u16			efi_type;	/* efi log item type */
+	__u16			efi_size;	/* size of this item */
+	__u32			efi_nextents;	/* # extents to free */
+	__u64			efi_id;		/* efi identifier */
+	xfs_extent_32_t		efi_extents[1];	/* array of extents to free */
+} xfs_efi_log_format_32_t;
+
+typedef struct xfs_efd_log_format_32 {
+	__u16			efd_type;	/* efd log item type */
+	__u16			efd_size;	/* size of this item */
+	__u32			efd_nextents;	/* # of extents freed */
+	__u64			efd_efi_id;	/* id of corresponding efi */
+	xfs_extent_32_t		efd_extents[1];	/* array of extents freed */
+} xfs_efd_log_format_32_t;
+
+#pragma pack 0
+/* end of packed stuctures */
 
 #include <sys/endian.h>
 #define __BYTE_ORDER	BYTE_ORDER
 #define __BIG_ENDIAN	BIG_ENDIAN
 #define __LITTLE_ENDIAN	LITTLE_ENDIAN
-#define __fswab16(x)	(x)
-#define __fswab32(x)	(x)
-#define __fswab64(x)	(x)
+#define __swab16(x)	(x)
+#define __swab32(x)	(x)
+#define __swab64(x)	(x)
 
 /* Map some gcc macros for the MipsPRO compiler */
 #ifndef __GNUC__
+#define INT_SWAP16(type,var)	(var)
+#define INT_SWAP32(type,var)	(var)
+#define INT_SWAP64(type,var)	(var)
+#define HAVE_SWABMACROS		1 /* no typeof available */
+#define __builtin_constant_p(x)	(0)
+#define __FUNCTION__	"XFS"
 #define __sgi__		__sgi
 #define __inline__	__inline
 #define inline		__inline
@@ -236,7 +276,7 @@ static __inline__ int platform_test_xfs_fd(int fd)
 	struct statvfs sbuf;
 	if (fstatvfs(fd, &sbuf) < 0)
 		return 0;
-	return (strcmp(sbuf.f_basetype, "xfs") == 0);
+	return strncmp(sbuf.f_basetype, "xfs", 4) == 0;
 }
 
 static __inline__ int platform_test_xfs_path(const char *path)
@@ -244,7 +284,7 @@ static __inline__ int platform_test_xfs_path(const char *path)
 	struct statvfs sbuf;
 	if (statvfs(path, &sbuf) < 0)
 		return 0;
-	return (strcmp(sbuf.f_basetype, "xfs") == 0);
+	return strncmp(sbuf.f_basetype, "xfs", 4) == 0;
 }
 
 static __inline__ int platform_fstatfs(int fd, struct statfs *buf)
@@ -257,6 +297,53 @@ static __inline__ void platform_getoptreset(void)
 	getoptreset();
 }
 
+static __inline__ int platform_uuid_compare(uuid_t *uu1, uuid_t *uu2)
+{
+	uint_t status;
+	return uuid_compare(uu1, uu2, &status);
+}
+
+static __inline__ void platform_uuid_unparse(uuid_t *uu, char *buffer)
+{
+	uint_t status;
+	char *s;
+	uuid_to_string(uu, &s, &status);
+	if (status == uuid_s_ok)
+		strcpy(buffer, s);
+	else buffer[0] = '\0';
+	free(s);
+}
+
+static __inline__ int platform_uuid_parse(char *buffer, uuid_t *uu)
+{
+	uint_t status;
+	uuid_from_string(buffer, uu, &status);
+	return (status == uuid_s_ok);
+}
+
+static __inline__ int platform_uuid_is_null(uuid_t *uu)
+{
+	uint status;
+	return uuid_is_nil(uu, &status);
+}
+
+static __inline__ void platform_uuid_generate(uuid_t *uu)
+{
+	uint_t status;
+	uuid_create(uu, &status);
+}
+
+static __inline__ void platform_uuid_clear(uuid_t *uu)
+{
+	uint_t status;
+	uuid_create_nil(uu, &status);
+}
+
+static __inline__ void platform_uuid_copy(uuid_t *dst, uuid_t *src)
+{
+	memcpy(dst, src, sizeof(uuid_t));
+}
+
 static __inline__ char * strsep(char **s, const char *ct)
 {
 	char *sbegin = *s, *end;
@@ -324,4 +411,8 @@ static __inline__ char * strsep(char **s, const char *ct)
 #define XFS_IOC_GETPARENTS		SGI_XFS_GETPARENTS
 #define XFS_IOC_GETPARENTPATHS		SGI_XFS_GETPARENTPATHS
 
+#define	_AIOCB64_T_DEFINED		1
+
+#define	XFS_XFLAG_NODEFRAG		0x00002000
+
 #endif	/* __XFS_IRIX_H__ */
