@@ -20,9 +20,6 @@
 #include <sys/stat.h>
 #include "init.h"
 
-#define findrawpath(x)	x
-#define findblockpath(x) x
-
 char *progname = "libxfs";	/* default, changed by each tool */
 
 struct cache *libxfs_icache;	/* global inode cache */
@@ -92,16 +89,22 @@ libxfs_device_open(char *path, int creat, int xflags, int setblksize)
 {
 	dev_t		dev;
 	int		fd, d, flags;
-	int		readonly, excl;
+	int		readonly, dio, excl;
 	struct stat64	statb;
 
 	readonly = (xflags & LIBXFS_ISREADONLY);
 	excl = (xflags & LIBXFS_EXCLUSIVELY) && !creat;
+	dio = (xflags & LIBXFS_DIRECT) && !creat && platform_direct_blockdev();
 
+retry:
 	flags = (readonly ? O_RDONLY : O_RDWR) | \
 		(creat ? (O_CREAT|O_TRUNC) : 0) | \
+		(dio ? O_DIRECT : 0) | \
 		(excl ? O_EXCL : 0);
+
 	if ((fd = open(path, flags, 0666)) < 0) {
+		if (errno == EINVAL && --dio == 0)
+			goto retry;
 		fprintf(stderr, _("%s: cannot open %s: %s\n"),
 			progname, path, strerror(errno));
 		exit(1);
@@ -113,9 +116,8 @@ libxfs_device_open(char *path, int creat, int xflags, int setblksize)
 		exit(1);
 	}
 
-	if (!readonly && setblksize && (statb.st_mode & S_IFMT) == S_IFBLK) {
-		platform_set_blocksize(fd, path, 512);
-	}
+	if (!readonly && setblksize && (statb.st_mode & S_IFMT) == S_IFBLK)
+		platform_set_blocksize(fd, path, statb.st_rdev, 512);
 
 	/*
 	 * Get the device number from the stat buf - unless
@@ -181,13 +183,13 @@ check_open(char *path, int flags, char **rawfile, char **blockfile)
 		perror(path);
 		return 0;
 	}
-	if (!(*rawfile = findrawpath(path))) {
+	if (!(*rawfile = platform_findrawpath(path))) {
 		fprintf(stderr, _("%s: "
 				  "can't find a character device matching %s\n"),
 			progname, path);
 		return 0;
 	}
-	if (!(*blockfile = findblockpath(path))) {
+	if (!(*blockfile = platform_findblockpath(path))) {
 		fprintf(stderr, _("%s: "
 				  "can't find a block device matching %s\n"),
 			progname, path);
@@ -235,87 +237,16 @@ libxfs_init(libxfs_init_t *a)
 	(void)getcwd(curdir,MAXPATHLEN);
 	needcd = 0;
 	fd = -1;
-	flags = a->isreadonly;
+	flags = (a->isreadonly | a->isdirect);
 
 	if (a->volname) {
 		if(!check_open(a->volname,flags,&rawfile,&blockfile))
 			goto done;
 		needcd = 1;
 		fd = open(rawfile, O_RDONLY);
-#ifdef HAVE_VOLUME_MANAGER
-		xlv_getdev_t getdev;
-		if (ioctl(fd, DIOCGETVOLDEV, &getdev) < 0)
-#else
-		if (1)
-#endif
-		{
-			if (a->notvolok) {
-				dname = a->dname = a->volname;
-				a->volname = NULL;
-				goto voldone;
-			}
-			fprintf(stderr, _("%s: "
-				"%s is not a volume device name\n"),
-				progname, a->volname);
-			if (a->notvolmsg)
-				fprintf(stderr, a->notvolmsg, a->volname);
-			goto done;
-		}
-#ifdef HAVE_VOLUME_MANAGER
-		if (getdev.data_subvol_dev && dname) {
-			fprintf(stderr, _("%s: "
-				"%s has a data subvolume, cannot specify %s\n"),
-				progname, a->volname, dname);
-			goto done;
-		}
-		if (getdev.log_subvol_dev && logname) {
-			fprintf(stderr, _("%s: "
-				"%s has a log subvolume, cannot specify %s\n"),
-				progname, a->volname, logname);
-			goto done;
-		}
-		if (getdev.rt_subvol_dev && rtname) {
-			fprintf(stderr, _("%s: %s has a realtime subvolume, "
-				"cannot specify %s\n"),
-				progname, a->volname, rtname);
-			goto done;
-		}
-		if (!dname && getdev.data_subvol_dev) {
-			strcpy(dpath, "/tmp/libxfsdXXXXXX");
-			(void)mktemp(dpath);
-			if (mknod(dpath, S_IFCHR | 0600,
-				  getdev.data_subvol_dev) < 0) {
-				fprintf(stderr, _("%s: mknod failed: %s\n"),
-					progname, strerror(errno));
-				goto done;
-			}
-			dname = dpath;
-		}
-		if (!logname && getdev.log_subvol_dev) {
-			strcpy(logpath, "/tmp/libxfslXXXXXX");
-			(void)mktemp(logpath);
-			if (mknod(logpath, S_IFCHR | 0600,
-				  getdev.log_subvol_dev) < 0) {
-				fprintf(stderr, _("%s: mknod failed: %s\n"),
-					progname, strerror(errno));
-				goto done;
-			}
-			logname = logpath;
-		}
-		if (!rtname && getdev.rt_subvol_dev) {
-			strcpy(rtpath, "/tmp/libxfsrXXXXXX");
-			(void)mktemp(rtpath);
-			if (mknod(rtpath, S_IFCHR | 0600,
-				  getdev.rt_subvol_dev) < 0) {
-				fprintf(stderr, _("%s: mknod failed: %s\n"),
-					progname, strerror(errno));
-				goto done;
-			}
-			rtname = rtpath;
-		}
-#endif
+		dname = a->dname = a->volname;
+		a->volname = NULL;
 	}
-voldone:
 	if (dname) {
 		if (dname[0] != '/' && needcd)
 			chdir(curdir);
@@ -753,6 +684,12 @@ libxfs_destroy(void)
 	cache_destroy(libxfs_bcache);
 }
 
+int
+libxfs_device_alignment(void)
+{
+	return platform_align_blockdev();
+}
+
 void
 libxfs_report(FILE *fp)
 {
@@ -766,9 +703,3 @@ libxfs_report(FILE *fp)
 	c = asctime(localtime(&t));
 	fprintf(fp, "%s", c);
 }
-
-char *
-libxfs_findrawpath(char *path)
-{
-	return platform_findrawpath(path);
-}
