@@ -27,6 +27,7 @@
 #include <sys/ioctl.h>
 
 extern char *progname;
+static int max_block_alignment;
 
 #ifndef BLKGETSIZE64
 # define BLKGETSIZE64	_IOR(0x12,114,size_t)
@@ -102,12 +103,14 @@ platform_check_iswritable(char *name, char *block, struct stat64 *s, int fatal)
 }
 
 void
-platform_set_blocksize(int fd, char *path, int blocksize)
+platform_set_blocksize(int fd, char *path, dev_t device, int blocksize)
 {
-	if (ioctl(fd, BLKBSZSET, &blocksize) < 0) {
-		fprintf(stderr, _("%s: warning - cannot set blocksize "
-				"on block device %s: %s\n"),
-			progname, path, strerror(errno));
+	if (major(device) != RAMDISK_MAJOR) {
+		if (ioctl(fd, BLKBSZSET, &blocksize) < 0) {
+			fprintf(stderr, _("%s: warning - cannot set blocksize "
+					"on block device %s: %s\n"),
+				progname, path, strerror(errno));
+		}
 	}
 }
 
@@ -134,6 +137,8 @@ platform_findsizes(char *path, int fd, long long *sz, int *bsz)
 	if ((st.st_mode & S_IFMT) == S_IFREG) {
 		*sz = (long long)(st.st_size >> 9);
 		*bsz = BBSIZE;
+		if (BBSIZE > max_block_alignment)
+			max_block_alignment = BBSIZE;
 		return;
 	}
 
@@ -160,6 +165,8 @@ platform_findsizes(char *path, int fd, long long *sz, int *bsz)
 			progname, path, strerror(errno));
 		*bsz = BBSIZE;
 	}
+	if (*bsz > max_block_alignment)
+		max_block_alignment = *bsz;
 }
 
 int
@@ -178,11 +185,25 @@ platform_aio_init(int aio_count)
 char *
 platform_findrawpath(char *path)
 {
-	return (path);
+	return path;
 }
 
-size_t
-platform_memalignment(void)
+char *
+platform_findblockpath(char *path)
 {
-	return getpagesize();
+	return path;
+}
+
+int
+platform_direct_blockdev(void)
+{
+	return 1;
+}
+
+int
+platform_align_blockdev(void)
+{
+	if (!max_block_alignment)
+		return getpagesize();
+	return max_block_alignment;
 }
