@@ -126,10 +126,8 @@ xfs_init(libxfs_init_t *args)
 		/* XXX assume data file also means rt file */
 	}
 
-	args->notvolmsg = _("you should never get this message - %s");
-	args->notvolok = 1;
 	args->setblksize = !dangerously;
-
+	args->isdirect = LIBXFS_DIRECT;
 	if (no_modify)
 		args->isreadonly = (LIBXFS_ISREADONLY | LIBXFS_ISINACTIVE);
 	else if (dangerously)
