@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <libxfs.h>
@@ -81,7 +67,7 @@ lf_block_delete_orphanage(xfs_mount_t		*mp,
 	for (i = 0; i < INT_GET(leaf->hdr.count, ARCH_CONVERT); entry++, i++) {
 		namest = XFS_DIR_LEAF_NAMESTRUCT(leaf,
 			INT_GET(entry->nameidx, ARCH_CONVERT));
-		XFS_DIR_SF_GET_DIRINO_ARCH(&namest->inumber, &lino, ARCH_CONVERT);
+		XFS_DIR_SF_GET_DIRINO(&namest->inumber, &lino);
 		bcopy(namest->name, fname, entry->namelen);
 		fname[entry->namelen] = '\0';
 
@@ -316,7 +302,7 @@ shortform_delete_orphanage(xfs_mount_t	*mp,
 	res = 0;
 	irec = NULL;
 	ino_dir_size = INT_GET(root_dino->di_core.di_size, ARCH_CONVERT);
-	max_size = XFS_DFORK_DSIZE_ARCH(root_dino, mp, ARCH_CONVERT);
+	max_size = XFS_DFORK_DSIZE(root_dino, mp);
 	use_rbuf = 0;
 	root_agno = XFS_INO_TO_AGNO(mp, mp->m_sb.sb_rootino);
 	root_agino = XFS_INO_TO_AGINO(mp, mp->m_sb.sb_rootino);
@@ -329,8 +315,7 @@ shortform_delete_orphanage(xfs_mount_t	*mp,
 			(__psint_t)next_sfe - (__psint_t)sf; i++)  {
 		tmp_sfe = NULL;
 		sf_entry = next_sfe;
-		XFS_DIR_SF_GET_DIRINO_ARCH(&sf_entry->inumber,
-			&lino, ARCH_CONVERT);
+		XFS_DIR_SF_GET_DIRINO(&sf_entry->inumber, &lino);
 		bcopy(sf_entry->name, fname, sf_entry->namelen);
 		fname[sf_entry->namelen] = '\0';
 
@@ -526,7 +511,7 @@ lf2_block_delete_orphanage(xfs_mount_t		*mp,
 	ptr = (char *)data->u;
 	if (INT_GET(data->hdr.magic, ARCH_CONVERT) == XFS_DIR2_BLOCK_MAGIC) {
 		btp = XFS_DIR2_BLOCK_TAIL_P(mp, (xfs_dir2_block_t *)data);
-		endptr = (char *)XFS_DIR2_BLOCK_LEAF_P_ARCH(btp, ARCH_CONVERT);
+		endptr = (char *)XFS_DIR2_BLOCK_LEAF_P(btp);
 	} else
 		endptr = (char *)data + mp->m_dirblksize;
 	*dirty = 0;
@@ -796,7 +781,7 @@ shortform2_delete_orphanage(xfs_mount_t	*mp,
 	*ino_dirty = 0;
 	irec = NULL;
 	ino_dir_size = INT_GET(root_dino->di_core.di_size, ARCH_CONVERT);
-	max_size = XFS_DFORK_DSIZE_ARCH(root_dino, mp, ARCH_CONVERT);
+	max_size = XFS_DFORK_DSIZE(root_dino, mp);
 	use_rbuf = 0;
 	res = 0;
 	root_agno = XFS_INO_TO_AGNO(mp, mp->m_sb.sb_rootino);
@@ -810,8 +795,8 @@ shortform2_delete_orphanage(xfs_mount_t	*mp,
 			(__psint_t)next_sfe - (__psint_t)sf; i++)  {
 		tmp_sfe = NULL;
 		sf_entry = next_sfe;
-		lino = XFS_DIR2_SF_GET_INUMBER_ARCH(sf,
-			XFS_DIR2_SF_INUMBERP(sf_entry), ARCH_CONVERT);
+		lino = XFS_DIR2_SF_GET_INUMBER(sf,
+			XFS_DIR2_SF_INUMBERP(sf_entry));
 		bcopy(sf_entry->name, fname, sf_entry->namelen);
 		fname[sf_entry->namelen] = '\0';
 
@@ -1191,6 +1176,21 @@ phase4(xfs_mount_t *mp)
 		 */
 		for (j = ag_hdr_block; j < ag_end; j++)  {
 
+			/* Process in chunks of 16 (XR_BB_UNIT/XR_BB) */
+			if ((extent_start == 0) && ((j & XR_BB_MASK) == 0)) {
+				switch(ba_bmap[i][j>>XR_BB]) {
+				case XR_E_UNKNOWN_LL:
+				case XR_E_FREE1_LL:
+				case XR_E_FREE_LL:
+				case XR_E_INUSE_LL:
+				case XR_E_INUSE_FS_LL:
+				case XR_E_INO_LL:
+				case XR_E_FS_MAP_LL:
+					j += (XR_BB_UNIT/XR_BB) - 1;
+					continue;
+				}
+			}
+
 			bstate = get_agbno_state(mp, i, j);
 
 			switch (bstate)  {
@@ -1301,13 +1301,11 @@ phase4(xfs_mount_t *mp)
 	 * initialize bitmaps for all AGs
 	 */
 	for (i = 0; i < mp->m_sb.sb_agcount; i++)  {
-		ag_end = (i < mp->m_sb.sb_agcount - 1) ? mp->m_sb.sb_agblocks :
-			mp->m_sb.sb_dblocks -
-				(xfs_drfsbno_t) mp->m_sb.sb_agblocks * i;
 		/*
 		 * now reset the bitmap for all ags
 		 */
-		bzero(ba_bmap[i], roundup(mp->m_sb.sb_agblocks/(NBBY/XR_BB),
+		bzero(ba_bmap[i], 
+		    roundup((mp->m_sb.sb_agblocks+(NBBY/XR_BB)-1)/(NBBY/XR_BB),
 						sizeof(__uint64_t)));
 		for (j = 0; j < ag_hdr_block; j++)
 			set_agbno_state(mp, i, j, XR_E_INUSE_FS);
